import { createConsumer } from '@rails/actioncable';
import { reactive, ref, watch } from "vue";
import Action from "../../action";

// Handler implementation
class SocketHandler {
  constructor(options) {
    this.targetId = options.targetId;
  }

  initialized() {
    console.log(`${this.targetId} initialized`);
  }

  rejected() {
    console.error(`${this.targetId} rejected`);
  }

  received(data) {
    Action.execute(data, this.component);
  }

  connected() {
    console.log(`${this.targetId} connected`);
  }

  disconnected() {
    console.log(`${this.targetId} disconnected`);
  }

  get component() {
    if (this.targetId) {
      return GLib.component.findById(this.targetId);
    }
  }
}
class ExampleSocketHandler extends SocketHandler {
  received(data) {
    this.component.text = data.text;
  }
}

const channels = reactive({});

// you can implement your own handler or just use the default one
const socketHandlers = reactive({
  ExampleSocketHandler
});

// API
const useSocket = (config, defaultConsumer = null) => {
  const consumer = defaultConsumer || createConsumer();
  consumer.connect();

  const onOpen = () => {
    if (!config.onOpen) return;
    Action.execute(config.onOpen, {});
  };
  const onClose = () => {
    if (!config.onClose) return;
    Action.execute(config.onClose, {});
  };
  const onError = () => {
    if (!config.onError) return;
    Action.execute(config.onError, {});
  };

  const readyState = ref(null);
  const timerId = setInterval(() => {
    readyState.value = consumer.connection.webSocket.readyState;
  }, 250);
  consumer.connection.events.error = () => {
    onError();
  };
  watch(readyState, (val, oldVal) => {
    if (val == window.WebSocket.OPEN && oldVal != window.WebSocket.OPEN) {
      onOpen();
    } else if (val == window.WebSocket.CLOSED && oldVal != window.WebSocket.CLOSED) {
      onClose();
    }
  });

  consumer.removeCustomListener = () => {
    clearInterval(timerId);
  };

  config.channels.forEach(ch => {
    if (!channels[ch.name]) {
      let handler = null;

      if (ch.handler && socketHandlers[ch.handler]) {
        handler = new socketHandlers[ch.handler](ch.options);
      } else {
        handler = new SocketHandler(ch.options);
      }

      const channel = consumer.subscriptions.create(
        { channel: ch.name, ...ch.options },
        {
          received(data) {
            handler.received(data);
          },
          initialized() {
            handler.initialized();
          },
          connected() {
            handler.connected();
          },
          rejected() {
            handler.rejected();
          },
          disconnected({ willAttemptReconnect: boolean }) {
            handler.disconnected();
          }
        }
      );
      channels[ch.name] = channel;
    }
  });

  return { consumer };
};

export { useSocket, channels };